@@ -5,7 +5,7 @@ ${CXX:-g++} -fprofile-arcs -ftest-coverage -fPIC -O0 example.cpp -o program
 ./program
 
 #BEGIN gcovr
-gcovr -r . --json-summary-pretty --json-summary
+gcovr --json-summary-pretty --json-summary
 #END gcovr
 
 rm -f program *.gc*
