@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
+# This file is part of gcovr 7.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2023 the gcovr authors
+# Copyright (c) 2013-2024 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -19,6 +19,7 @@
 
 from ..__main__ import main
 from ..version import __version__
+from ..formats.json.versions import JSON_FORMAT_VERSION
 
 import logging
 import pytest
@@ -105,24 +106,27 @@ def test_empty_exclude(capsys):
 
 
 def test_empty_exclude_directories(capsys):
-    c = capture(capsys, ["--exclude-directories", ""])
+    c = capture(capsys, ["--gcov-exclude-directories", ""])
     assert c.out == ""
     assert "filter cannot be empty" in c.err
     assert c.exception.code != 0
 
 
 def test_empty_objdir(capsys):
-    c = capture(capsys, ["--object-directory", ""])
+    c = capture(capsys, ["--gcov-object-directory", ""])
     assert c.out == ""
-    assert "argument --object-directory: Should not be set to an empty string." in c.err
+    assert (
+        "argument --gcov-object-directory/--object-directory: Should not be set to an empty string."
+        in c.err
+    )
     assert c.exception.code != 0
 
 
 def test_invalid_objdir(caplog):
-    c = log_capture(caplog, ["--object-directory", "not-existing-dir"])
+    c = log_capture(caplog, ["--gcov-object-directory", "not-existing-dir"])
     message = c.record_tuples[0]
     assert message[1] == logging.ERROR
-    assert message[2].startswith("Bad --object-directory option.")
+    assert message[2].startswith("Bad --gcov-object-directory option.")
     assert c.exception.code == 1
 
 
@@ -212,7 +216,7 @@ def test_non_existing_directory_2_csv(capsys):
     helper_test_non_existing_directory_2_output(capsys, "--csv")
 
 
-def helper_test_non_writable_directory_output(capsys, option):  # pragma: no cover
+def helper_test_non_writable_directory_output(capsys, option):
     c = capture(capsys, [option, "/file.txt"])
     assert c.out == ""
     assert "Could not create output file '/file.txt': " in c.err
@@ -220,47 +224,47 @@ def helper_test_non_writable_directory_output(capsys, option):  # pragma: no cov
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_output(capsys):  # pragma: no cover
+def test_non_writable_directory_output(capsys):
     helper_test_non_writable_directory_output(capsys, "--output")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_txt(capsys):  # pragma: no cover
+def test_non_writable_directory_txt(capsys):
     helper_test_non_writable_directory_output(capsys, "--txt")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_xml(capsys):  # pragma: no cover
+def test_non_writable_directory_xml(capsys):
     helper_test_non_writable_directory_output(capsys, "--xml")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_html(capsys):  # pragma: no cover
+def test_non_writable_directory_html(capsys):
     helper_test_non_writable_directory_output(capsys, "--html")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_html_details(capsys):  # pragma: no cover
+def test_non_writable_directory_html_details(capsys):
     helper_test_non_writable_directory_output(capsys, "--html-details")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_html_nested(capsys):  # pragma: no cover
+def test_non_writable_directory_html_nested(capsys):
     helper_test_non_writable_directory_output(capsys, "--html-nested")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_sonarqube(capsys):  # pragma: no cover
+def test_non_writable_directory_sonarqube(capsys):
     helper_test_non_writable_directory_output(capsys, "--sonarqube")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_json(capsys):  # pragma: no cover
+def test_non_writable_directory_json(capsys):
     helper_test_non_writable_directory_output(capsys, "--json")
 
 
 @pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
-def test_non_writable_directory_csv(capsys):  # pragma: no cover
+def test_non_writable_directory_csv(capsys):
     helper_test_non_writable_directory_output(capsys, "--csv")
 
 
@@ -286,27 +290,62 @@ def test_no_output_html_nested(caplog):
     assert c.exception.code != 0
 
 
-def test_branch_threshold_nan(capsys):
-    c = capture(capsys, ["--fail-under-branch", "nan"])
+@pytest.mark.parametrize(
+    "option",
+    [
+        "--fail-under-line",
+        "--fail-under-branch",
+        "--fail-under-decision",
+        "--fail-under-function",
+    ],
+)
+def test_failed_under_threshold_nan(option, capsys):
+    c = capture(capsys, [option, "nan"])
     assert c.out == ""
     assert "not in range [0.0, 100.0]" in c.err
     assert c.exception.code != 0
 
 
-def test_line_threshold_negative(capsys):
-    c = capture(capsys, ["--fail-under-line", "-0.1"])
+@pytest.mark.parametrize(
+    "option",
+    [
+        "--fail-under-line",
+        "--fail-under-branch",
+        "--fail-under-decision",
+        "--fail-under-function",
+    ],
+)
+def test_failed_under_threshold_negative(option, capsys):
+    c = capture(capsys, [option, "-0.1"])
     assert c.out == ""
     assert "not in range [0.0, 100.0]" in c.err
     assert c.exception.code != 0
 
 
-def test_line_threshold_100_1(capsys):
-    c = capture(capsys, ["--fail-under-line", "100.1"])
+@pytest.mark.parametrize(
+    "option",
+    [
+        "--fail-under-line",
+        "--fail-under-branch",
+        "--fail-under-decision",
+        "--fail-under-function",
+    ],
+)
+def test_failed_under_threshold_100_1(option, capsys):
+    c = capture(capsys, [option, "100.1"])
     assert c.out == ""
     assert "not in range [0.0, 100.0]" in c.err
     assert c.exception.code != 0
 
 
+def test_failed_under_decision_without_active_decision(caplog):
+    c = log_capture(caplog, ["--fail-under-decision", "90"])
+    message0 = c.record_tuples[0]
+    assert message0[1] == logging.ERROR
+    assert message0[2] == "--fail-under-decision need also option --decision."
+    assert c.exception.code != 0
+
+
 def test_filter_backslashes_are_detected(caplog):
     # gcov-exclude all to prevent any coverage data from being found
     c = log_capture(
@@ -343,7 +382,7 @@ def test_html_title_empty_string(caplog):
     c = log_capture(caplog, ["--html-title", ""])
     message = c.record_tuples[0]
     assert message[1] == logging.ERROR
-    assert message[2] == "an empty --html_title= is not allowed."
+    assert message[2] == "an empty --html-title= is not allowed."
     assert c.exception.code != 0
 
 
@@ -404,54 +443,89 @@ def test_html_tab_size_zero(caplog):
     assert c.exception.code != 0
 
 
+def test_html_template_dir(capsys):
+    c = capture(capsys, ["--html", "--html-template-dir", "foo"])
+    assert "<html" in c.out
+    assert "</html>" in c.out
+
+
 def test_multiple_output_formats_to_stdout(caplog):
     c = log_capture(
-        caplog, ["--xml", "--html", "--sonarqube", "--coveralls", "--root", "gcovr"]
-    )
-    message = c.record_tuples[0]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "HTML output skipped - consider providing an output file with `--html=OUTPUT`."
-    )
-    message = c.record_tuples[1]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "Sonarqube output skipped - consider providing an output file with `--sonarqube=OUTPUT`."
-    )
-    message = c.record_tuples[2]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "Coveralls output skipped - consider providing an output file with `--coveralls=OUTPUT`."
+        caplog,
+        [
+            "--coveralls",
+            "--cobertura",
+            "--csv",
+            "--html",
+            "--jacoco",
+            "--json",
+            "--json-summary",
+            "--sonarqube",
+            "--txt",
+            "--root",
+            "gcovr",
+        ],
     )
+    for index, text_fragments in enumerate(
+        [
+            ("Coveralls", "--coveralls"),
+            ("CSV", "--csv"),
+            ("HTML", "--html"),
+            ("JaCoCo", "--jacoco"),
+            ("JSON", "--json"),
+            ("JSON summary", "--json-summary"),
+            ("SonarQube", "--sonarqube"),
+            ("Text", "--txt"),
+        ]
+    ):
+        format, option = text_fragments
+        message = c.record_tuples[index]
+        assert message[1] == logging.WARNING
+        assert (
+            message[2]
+            == f"{format} output skipped - consider providing an output file with `{option}=OUTPUT`."
+        )
     assert c.exception.code == 0
 
 
 def test_multiple_output_formats_to_stdout_1(caplog):
     c = log_capture(
         caplog,
-        ["--xml", "--html", "--sonarqube", "--coveralls", "-o", "-", "--root", "gcovr"],
-    )
-    message = c.record_tuples[0]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "HTML output skipped - consider providing an output file with `--html=OUTPUT`."
-    )
-    message = c.record_tuples[1]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "Sonarqube output skipped - consider providing an output file with `--sonarqube=OUTPUT`."
-    )
-    message = c.record_tuples[2]
-    assert message[1] == logging.WARNING
-    assert (
-        message[2]
-        == "Coveralls output skipped - consider providing an output file with `--coveralls=OUTPUT`."
+        [
+            "--coveralls",
+            "--cobertura",
+            "--csv",
+            "--html",
+            "--jacoco",
+            "--json",
+            "--json-summary",
+            "--sonarqube",
+            "--txt",
+            "-o",
+            "-",
+            "--root",
+            "gcovr",
+        ],
     )
+    for index, text_fragments in enumerate(
+        [
+            ("Coveralls", "--coveralls"),
+            ("CSV", "--csv"),
+            ("HTML", "--html"),
+            ("JaCoCo", "--jacoco"),
+            ("JSON", "--json"),
+            ("JSON summary", "--json-summary"),
+            ("SonarQube", "--sonarqube"),
+            ("Text", "--txt"),
+        ]
+    ):
+        format, option = text_fragments
+        message = c.record_tuples[index]
+        assert message[1] == logging.WARNING
+        assert (
+            message[2]
+            == f"{format} output skipped - consider providing an output file with `{option}=OUTPUT`."
+        )
     assert c.exception.code == 0
 
 
@@ -472,7 +546,7 @@ def test_html_injection_via_json(capsys, tmp_path):
 
     script = '<script>alert("pwned")</script>'
     jsondata = {
-        "gcovr/format_version": "0.5",
+        "gcovr/format_version": JSON_FORMAT_VERSION,
         "files": [
             {"file": script, "functions": [], "lines": []},
             {"file": "other", "functions": [], "lines": []},
@@ -491,6 +565,125 @@ def test_html_injection_via_json(capsys, tmp_path):
     assert c.exception.code == 0
 
 
+def test_import_valid_cobertura_file(tmp_path):
+    from ..formats import read_reports
+    from ..configuration import merge_options_and_set_defaults
+    from ..coverage import FileCoverage
+
+    testfile = "/path/to/source/code.cpp"
+    xmldata = f"""<?xml version='1.0' encoding='UTF-8'?>
+<!DOCTYPE coverage SYSTEM 'http://cobertura.sourceforge.net/xml/coverage-04.dtd'>
+<coverage line-rate="0.9" branch-rate="0.75" lines-covered="9" lines-valid="10" branches-covered="3" branches-valid="4" complexity="0.0" timestamp="" version="gcovr 7.0">
+  <sources>
+    <source>.</source>
+  </sources>
+  <packages>
+    <package name="source" line-rate="0.9" branch-rate="0.75" complexity="0.0">
+      <classes>
+        <class name="code_cpp" filename="{testfile}" line-rate="0.9" branch-rate="0.75" complexity="0.0">
+          <methods/>
+          <lines>
+            <line number="3" hits="3" branch="false"/>
+            <line number="4" hits="3" branch="true" condition-coverage="100% (2/2)">
+              <conditions>
+                <condition number="0" type="jump" coverage="100%"/>
+              </conditions>
+            </line>
+            <line number="5" hits="2" branch="false"/>
+            <line number="7" hits="1" branch="false"/>
+            <line number="9" hits="3" branch="false"/>
+            <line number="12" hits="2" branch="false"/>
+            <line number="13" hits="2" branch="true" condition-coverage="50% (1/2)">
+              <conditions>
+                <condition number="0" type="jump" coverage="50%"/>
+              </conditions>
+            </line>
+            <line number="14" hits="2" branch="false"/>
+            <line number="16" hits="0" branch="false"/>
+            <line number="18" hits="2" branch="false"/>
+          </lines>
+        </class>
+      </classes>
+    </package>
+  </packages>
+</coverage>
+    """
+    testfile = os.path.abspath(testfile)
+    tempfile = tmp_path / "valid_cobertura.xml"
+    with tempfile.open("w+") as fp:
+        fp.write(xmldata)
+
+    filename = str(tempfile)
+    opts = merge_options_and_set_defaults(
+        [{"cobertura_add_tracefile": [filename], "filter": [re.compile(".")]}]
+    )
+    covdata = read_reports(opts)
+    assert covdata is not None
+    assert testfile in covdata
+    fcov: FileCoverage = covdata[testfile]
+    assert len(fcov.lines) == 10
+    for line, count, branches in [
+        (7, 1, None),
+        (9, 3, None),
+        (16, 0, None),
+        (13, 2, [1, 0]),
+    ]:
+        assert fcov.lines[line].count == count
+        if branches is not None:
+            assert len(fcov.lines[line].branches) == len(branches)
+            for branch_idx, branch_count in enumerate(branches):
+                assert fcov.lines[line].branches[branch_idx].count == branch_count
+
+
+def test_import_corrupt_cobertura_file(caplog, tmp_path):
+    xmldata = "weiuh wliecsdfsef"
+    tempfile = tmp_path / "corrupt_cobertura.xml"
+    with tempfile.open("w+") as fp:
+        fp.write(xmldata)
+
+    c = log_capture(caplog, ["--cobertura-add-tracefile", str(tempfile)])
+    message = c.record_tuples[0]
+    assert message[1] == logging.ERROR
+    assert "Start tag expected" in message[2]
+    assert c.exception.code != 0
+
+
+def test_import_cobertura_file_with_invalid_line(caplog, tmp_path):
+    xmldata = """<?xml version='1.0' encoding='UTF-8'?>
+<!DOCTYPE coverage SYSTEM 'http://cobertura.sourceforge.net/xml/coverage-04.dtd'>
+<coverage line-rate="0.9" branch-rate="0.75" lines-covered="9" lines-valid="10" branches-covered="3" branches-valid="4" complexity="0.0" timestamp="" version="gcovr 7.0">
+  <sources>
+    <source>.</source>
+  </sources>
+  <packages>
+    <package name="source" line-rate="0.9" branch-rate="0.75" complexity="0.0">
+      <classes>
+        <class name="code_cpp" filename="/path/to/source/code.cpp" line-rate="0.9" branch-rate="0.75" complexity="0.0">
+          <methods/>
+          <lines>
+            <line number="3" hits="3" branch="false"/>
+            <line number="asdas" branch="true" condition-coverage="100% (2/2)" />
+            <line number="18" hits="2" branch="false"/>
+          </lines>
+        </class>
+      </classes>
+    </package>
+  </packages>
+</coverage>
+    """
+    tempfile = tmp_path / "cobertura_invalid_line.xml"
+    with tempfile.open("w+") as fp:
+        fp.write(xmldata)
+
+    c = log_capture(
+        caplog, ["--cobertura-add-tracefile", str(tempfile), "--filter", "."]
+    )
+    message = c.record_tuples[0]
+    assert message[1] == logging.ERROR
+    assert "'number' attribute is required and must be an integer" in message[2]
+    assert c.exception.code != 0
+
+
 def test_exclude_lines_by_pattern(caplog):
     c = log_capture(caplog, ["--exclude-lines-by-pattern", "example.**"])
     message = c.record_tuples[0]
@@ -516,3 +709,35 @@ def test_invalid_timestamp(capsys):
     assert c.out == ""
     assert "argument --timestamp: unknown timestamp format: 'foo'" in c.err
     assert c.exception.code != 0
+
+
+def test_sort_branch_and_not_uncovered_or_percent(caplog):
+    c = log_capture(caplog, ["--sort-branches"])
+    message = c.record_tuples[0]
+    assert message[1] == logging.ERROR
+    assert message[2].startswith(
+        "the options --sort-branches without '--sort uncovered-number' or '--sort uncovered-percent' doesn't make sense."
+    )
+    assert c.exception.code == 1
+
+
+@pytest.mark.parametrize(
+    "option",
+    [
+        ("-b", "--txt-metric branch"),
+        ("--txt-branches", "--txt-metric branch"),
+        ("--branches", "--txt-metric branch"),
+        ("--sort-uncovered", "--sort-key uncovered-number"),
+        ("--sort-percentage", "--sort-key uncovered-percent"),
+    ],
+    ids=lambda option: option[0],
+)
+def test_deprecated_option(caplog, option):
+    c = log_capture(caplog, [option[0]])
+    message = c.record_tuples[0]
+    assert message[1] == logging.WARNING
+    assert (
+        f"Deprecated option {option[0]} used, please use {option[1]!r} instead"
+        in message[2]
+    )
+    assert c.exception.code != 1
