@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
+# This file is part of gcovr 7.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2023 the gcovr authors
+# Copyright (c) 2013-2024 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -27,9 +27,9 @@ from unittest import mock
 
 import pytest
 
-from ..gcov_parser import NegativeHits, parse_coverage, UnknownLineType
+from ..formats.gcov.parser import NegativeHits, parse_coverage, UnknownLineType
 from ..utils import configure_logging
-from ..workers import Workers
+from ..formats.gcov.workers import Workers
 from ..exclusions import ExclusionOptions, apply_all_exclusions
 
 configure_logging()
@@ -439,7 +439,7 @@ def test_exception_during_coverage_processing(caplog):
     )
     lines = source.splitlines()
 
-    with mock.patch("gcovr.gcov_parser.insert_function_coverage") as m:
+    with mock.patch("gcovr.formats.gcov.parser.insert_function_coverage") as m:
         m.side_effect = AssertionError("totally broken")
         with pytest.raises(AssertionError) as ex_info:
             parse_coverage(
@@ -486,7 +486,7 @@ def test_trailing_function_tag():
     source = textwrap.dedent(
         """\
       #####: 2:example line
-    function example called 17 returned 16 blocks executed 3
+    function example called 17 returned 16 blocks executed 3%
     """
     )
 
@@ -582,7 +582,31 @@ def test_negativ_branch_count():
         )
 
 
-def test_negativ_branch_count_ignored():
+def test_negative_line_count_ignored():
+    """
+    A exception shall be raised.
+    """
+
+    source = textwrap.dedent(
+        """\
+             1: 1:foo += 1;
+            -1: 2:foo += 1;
+             2: 3:foo += 1;
+        """
+    )
+
+    coverage, _ = parse_coverage(
+        source.splitlines(),
+        filename="example.cpp",
+        ignore_parse_errors=set(["negative_hits.warn"]),
+    )
+
+    covered_lines = {line.lineno for line in coverage.lines.values() if line.is_covered}
+
+    assert covered_lines == {1, 3}
+
+
+def test_negative_branch_count_ignored():
     """
     A exception shall be raised.
     """
@@ -598,7 +622,7 @@ def test_negativ_branch_count_ignored():
         """
     )
 
-    coverage, lines = parse_coverage(
+    coverage, _ = parse_coverage(
         source.splitlines(),
         filename="example.cpp",
         ignore_parse_errors=set(["negative_hits.warn"]),
