@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
+# This file is part of gcovr 7.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2023 the gcovr authors
+# Copyright (c) 2013-2024 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -38,6 +38,7 @@ python_interpreter = force_unix_separator(
     sys.executable
 )  # use forward slash on windows as well
 env = os.environ
+env["SOURCE_DATE_EPOCH"] = "1706214941"
 env["GCOVR"] = python_interpreter + " -m gcovr"
 for var in [
     "CPATH",
@@ -60,7 +61,6 @@ skip_clean = None
 GCOVR_ISOLATED_TEST = os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"
 
 CC = os.path.split(env["CC"])[1]
-IS_CLANG = True if CC.startswith("clang") else False
 
 IS_MACOS = platform.system() == "Darwin"
 IS_WINDOWS = platform.system() == "Windows"
@@ -76,12 +76,14 @@ if IS_WINDOWS:
     env["GCOVR_TEST_DRIVE_WINDOWS"] = f"{free_drives[-1]}:"
 
 CC_REFERENCE = env.get("CC_REFERENCE", CC)
+CC_REFERENCE_VERSION = int(CC_REFERENCE.split("-")[1])
+IS_CLANG = True if CC_REFERENCE.startswith("clang") else False
 
 REFERENCE_DIRS = []
 REFERENCE_DIR_VERSION_LIST = (
-    ["gcc-5", "gcc-6", "gcc-8", "gcc-9", "gcc-10", "gcc-11"]
+    ["gcc-5", "gcc-6", "gcc-8", "gcc-9", "gcc-10", "gcc-11", "gcc-12", "gcc-13"]
     if "gcc" in CC_REFERENCE
-    else ["clang-10", "clang-13", "clang-14"]
+    else ["clang-10", "clang-13", "clang-14", "clang-15"]
 )
 for ref in REFERENCE_DIR_VERSION_LIST:
     REFERENCE_DIRS.append(os.path.join("reference", ref))
@@ -95,12 +97,11 @@ RE_DECIMAL = re.compile(r"(\d+\.\d+)")
 
 RE_TXT_WHITESPACE = re.compile(r"[ ]+$", flags=re.MULTILINE)
 
-RE_XML_ATTRS = re.compile(r'(timestamp)="[^"]*"')
-RE_XML_GCOVR_VERSION = re.compile(r'version="gcovr [^"]+"')
+RE_LCOV_PATH = re.compile(r"(SF:).+?/(gcovr/tests/.+?)$", flags=re.MULTILINE)
 
-RE_COVERALLS_CLEAN_KEYS = re.compile(
-    r'"(commit_sha|repo_token|run_at|version)": "[^"]*"'
-)
+RE_XML_ATTRS = re.compile(r'(timestamp)="[^"]*"')
+
+RE_COVERALLS_CLEAN_KEYS = re.compile(r'"(commit_sha|repo_token|run_at)": "[^"]*"')
 RE_COVERALLS_GIT = re.compile(
     r'"git": \{(?:"[^"]*": (?:"[^"]*"|\{[^\}]*\}|\[[^\]]*\])(?:, )?)+\}, '
 )
@@ -109,16 +110,19 @@ RE_COVERALLS_GIT_PRETTY = re.compile(
 )
 
 RE_HTML_ATTRS = re.compile('((timestamp)|(version))="[^"]*"')
-RE_HTML_FOOTER_VERSION = re.compile(
-    r"(Generated by: <a href=\".+/)(?:master|\d+\.\d+)(\">GCOVR \(Version )(?:master|\d+\.\d+)(\)</a>)"
+RE_HTML_HEADER_DATE = re.compile(
+    r"(<td)>\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d(?:\+\d\d:\d\d)?<(/td>)"
 )
-RE_HTML_HEADER_DATE = re.compile(r"(<td)>\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d<(/td>)")
 
 
 def scrub_txt(contents):
     return RE_TXT_WHITESPACE.sub("", contents)
 
 
+def scrub_lcov(contents):
+    return RE_LCOV_PATH.sub(r"\1\2", contents)
+
+
 def scrub_csv(contents):
     contents = force_unix_separator(contents)
     return contents
@@ -127,13 +131,11 @@ def scrub_csv(contents):
 def scrub_xml(contents):
     contents = RE_DECIMAL.sub(lambda m: str(round(float(m.group(1)), 5)), contents)
     contents = RE_XML_ATTRS.sub(r'\1=""', contents)
-    contents = RE_XML_GCOVR_VERSION.sub('version=""', contents)
     return contents
 
 
 def scrub_html(contents):
     contents = RE_HTML_ATTRS.sub('\\1=""', contents)
-    contents = RE_HTML_FOOTER_VERSION.sub("\\1x.x\\2x.x\\3", contents)
     contents = RE_HTML_HEADER_DATE.sub("\\1>0000-00-00 00:00:00<\\2", contents)
     contents = force_unix_separator(contents)
     return contents
@@ -231,6 +233,8 @@ KNOWN_FORMATS = [
     # Other formats
     "cobertura",
     "coveralls",
+    "jacoco",
+    "lcov",
     "sonarqube",
 ]
 
@@ -301,6 +305,14 @@ def pytest_generate_tests(metafunc):
                     name == "cmake_gtest" and not GCOVR_ISOLATED_TEST,
                     reason="only available in docker",
                 ),
+                pytest.mark.skipif(
+                    name == "gcov-no_working_dir_found" and not GCOVR_ISOLATED_TEST,
+                    reason="only available in docker",
+                ),
+                pytest.mark.xfail(
+                    name in ["gcov-ignore_output_error"] and IS_WINDOWS,
+                    reason="Permission is ignored on Windows",
+                ),
                 pytest.mark.xfail(
                     name == "exclude-throw-branches"
                     and format == "html"
@@ -313,14 +325,16 @@ def pytest_generate_tests(metafunc):
                 ),
                 pytest.mark.xfail(
                     name == "html-source-encoding-cp1252" and IS_CLANG,
-                    reason="clang doesnt understand -finput-charset=...",
+                    reason="clang doesn't understand -finput-charset=...",
                 ),
                 pytest.mark.xfail(
-                    name == "html-source-encoding-cp1252" and IS_MACOS,
-                    reason="On MacOS -finput-charset=cp1252 isn't supported",
-                ),
-                pytest.mark.xfail(
-                    name in ["excl-branch", "exclude-throw-branches", "html-themes"]
+                    name
+                    in [
+                        "excl-branch",
+                        "exclude-throw-branches",
+                        "html-themes",
+                        "html-themes-github",
+                    ]
                     and IS_MACOS,
                     reason="On MacOS the constructor is called twice",
                 ),
@@ -328,6 +342,11 @@ def pytest_generate_tests(metafunc):
                     name == "noncode" and IS_MACOS,
                     reason="On MacOS the there are other branches",
                 ),
+                pytest.mark.xfail(
+                    name == "decisions"
+                    and (IS_CLANG and CC_REFERENCE_VERSION == 15 and IS_MACOS),
+                    reason="On MacOS with clang 15 the file decision/switch_test.h throws compiler errors",
+                ),
                 pytest.mark.xfail(
                     name in ["decisions-neg-delta"] and IS_MACOS,
                     reason="On MacOS there is no branch for std::vector",
@@ -341,11 +360,7 @@ def pytest_generate_tests(metafunc):
                     reason="Only windows has a case insensitive file system",
                 ),
                 pytest.mark.xfail(
-                    name == "gcc-abspath"
-                    and (
-                        not env["CC"].startswith("gcc-")
-                        or int(env["CC"].replace("gcc-", "")) < 8
-                    ),
+                    name == "gcc-abspath" and (IS_CLANG or CC_REFERENCE_VERSION < 8),
                     reason="Option -fprofile-abs-path is supported since gcc-8",
                 ),
             ]
@@ -451,6 +466,8 @@ SCRUBBERS = dict(
     # Other formats
     cobertura=scrub_xml,
     coveralls=scrub_coveralls,
+    jacoco=scrub_xml,
+    lcov=scrub_lcov,
     sonarqube=scrub_xml,
 )
 
@@ -464,6 +481,8 @@ OUTPUT_PATTERN = dict(
     # Other formats
     cobertura=["cobertura*.xml"],
     coveralls=["coveralls*.json"],
+    jacoco=["jacoco*.xml"],
+    lcov=["coverage*.lcov"],
     sonarqube=["sonarqube*.xml"],
 )
 
@@ -485,7 +504,7 @@ def test_build(
         encoding = re.match("^html-encoding-(.*)$", name).group(1)
 
     os.chdir(os.path.join(basedir, name))
-    assert run(["make", format])
+    assert run(["make", "-j", "4", "--output-sync=target", format])
 
     if generate_reference:  # pragma: no cover
         generate_reference_data(output_pattern)
