@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
+# This file is part of gcovr 7.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2023 the gcovr authors
+# Copyright (c) 2013-2024 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -18,37 +18,43 @@
 # ****************************************************************************
 
 import glob
+import io
 import os
 import platform
 import re
 import shutil
-import shlex
 import subprocess
-import sys
+import zipfile
 import nox
 
 
-GCC_VERSIONS = [
+ALL_COMPILER_VERSIONS = [
     "gcc-5",
     "gcc-6",
     "gcc-8",
     "gcc-9",
     "gcc-10",
     "gcc-11",
+    "gcc-12",
+    "gcc-13",
     "clang-10",
     "clang-13",
     "clang-14",
+    "clang-15",
 ]
 
-GCC_VERSIONS_NEWEST_FIRST = [
+ALL_COMPILER_VERSIONS_NEWEST_FIRST = [
     "-".join(cc)
     for cc in sorted(
-        [(*cc.split("-"),) for cc in GCC_VERSIONS],
+        [(*cc.split("-"),) for cc in ALL_COMPILER_VERSIONS],
         key=lambda cc: (cc[0], int(cc[1])),
         reverse=True,
     )
 ]
 
+ALL_GCC_VERSIONS = [v for v in ALL_COMPILER_VERSIONS if v.startswith("gcc-")]
+ALL_CLANG_VERSIONS = [v for v in ALL_COMPILER_VERSIONS if v.startswith("clang-")]
+
 DEFAULT_TEST_DIRECTORIES = ["doc", "gcovr"]
 DEFAULT_LINT_ARGUMENTS = [
     "setup.py",
@@ -79,7 +85,7 @@ def get_gcc_version_to_use():
         return os.path.split(cc)[1]
 
     # Find the first insalled compiler version we suport
-    for cc in GCC_VERSIONS_NEWEST_FIRST:
+    for cc in ALL_COMPILER_VERSIONS_NEWEST_FIRST:
         if shutil.which(cc):
             return cc
 
@@ -125,6 +131,13 @@ def set_environment(session: nox.Session, cc: str, check: bool = True) -> None:
     session.env["CXXFLAGS"] = "--this_flag_does_not_exist"
 
 
+@nox.session
+def bump_version(session: nox.Session) -> None:
+    """Bump the new version"""
+    session.install("-e", ".")
+    session.run("python", "admin/bump_version.py")
+
+
 @nox.session(python=False)
 def qa(session: nox.Session) -> None:
     """Run the quality tests for the default GCC version."""
@@ -134,7 +147,7 @@ def qa(session: nox.Session) -> None:
 
 
 @nox.session(python=False)
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def qa_compiler(session: nox.Session, version: str) -> None:
     """Run the quality tests for a specific GCC version."""
     session_id = "lint"
@@ -148,26 +161,34 @@ def qa_compiler(session: nox.Session, version: str) -> None:
     session.notify(session_id)
 
 
-@nox.session
+@nox.session(python=False)
 def lint(session: nox.Session) -> None:
-    """Run the lint (flake8 and black)."""
-    session.install("flake8", "flake8-print")
+    """Run the linters."""
+    if session.posargs:
+        args = session.posargs
+    else:
+        args = DEFAULT_LINT_ARGUMENTS
+    session.notify("flake8", args)
+
     # Black installs under Pypy but doesn't necessarily run (cf psf/black#2559).
     if platform.python_implementation() == "CPython":
-        session.install(BLACK_PINNED_VERSION)
+        session.notify("black", ["--diff", "--check", *args])
+    else:
+        session.log(
+            f"Skip black because of platform {platform.python_implementation()}."
+        )
+
+
+@nox.session
+def flake8(session: nox.Session) -> None:
+    """Run flake8."""
+    session.install("flake8", "flake8-print")
     if session.posargs:
         args = session.posargs
     else:
         args = DEFAULT_LINT_ARGUMENTS
     session.run("flake8", *args)
 
-    if platform.python_implementation() == "CPython":
-        session.run("python", "-m", "black", "--diff", "--check", *args)
-    else:
-        session.log(
-            f"Skip black because of platform {platform.python_implementation()}."
-        )
-
 
 @nox.session
 def black(session: nox.Session) -> None:
@@ -205,21 +226,39 @@ def tests(session: nox.Session) -> None:
 
 @nox.session(python=False, name="tests_compiler(all)")
 def tests_compiler_all(session: nox.Session) -> None:
+    """Run the tests with all compiler versions."""
+    for version in ALL_COMPILER_VERSIONS:
+        session_id = f"tests_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="tests_compiler(gcc)")
+def tests_compiler_gcc(session: nox.Session) -> None:
     """Run the tests with all GCC versions."""
-    for version in GCC_VERSIONS:
+    for version in ALL_GCC_VERSIONS:
+        session_id = f"tests_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="tests_compiler(clang)")
+def tests_compiler_clang(session: nox.Session) -> None:
+    """Run the tests with all CLANG versions."""
+    for version in ALL_CLANG_VERSIONS:
         session_id = f"tests_compiler({version})"
         session.log(f"Notify session {session_id}")
         session.notify(session_id)
 
 
 @nox.session
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def tests_compiler(session: nox.Session, version: str) -> None:
     """Run the test with a specific GCC version."""
     session.install(
         "jinja2",
         "lxml",
-        "pygments==2.7.4",
+        "pygments==2.13.0",
         "pytest",
         "pytest-timeout",
         "cmake",
@@ -229,7 +268,7 @@ def tests_compiler(session: nox.Session, version: str) -> None:
         session.install("pywin32")
     coverage_args = []
     if os.environ.get("USE_COVERAGE") == "true":
-        session.install("pytest-cov")
+        session.install("coverage", "pytest-cov")
         coverage_args = ["--cov=gcovr", "--cov-branch"]
     session.install("-e", ".")
     set_environment(session, version)
@@ -256,13 +295,15 @@ def tests_compiler(session: nox.Session, version: str) -> None:
     if "--" not in args:
         args += ["--"] + DEFAULT_TEST_DIRECTORIES
     session.run("python", *args)
+    if os.environ.get("USE_COVERAGE") == "true":
+        session.run("coverage", "xml")
 
 
 @nox.session
 def build_wheel(session: nox.Session) -> None:
     """Build a wheel."""
-    session.install("wheel")
-    session.run("python", "setup.py", "sdist", "bdist_wheel")
+    session.install("build")
+    session.run("python", "-m", "build")
     dist_cache = f"{session.cache_dir}/dist"
     if os.path.isdir(dist_cache):
         shutil.rmtree(dist_cache)
@@ -294,7 +335,7 @@ def upload_wheel(session: nox.Session) -> None:
 @nox.session
 def bundle_app(session: nox.Session) -> None:
     """Bundle a standalone executable."""
-    session.install("pyinstaller")
+    session.install("pyinstaller~=5.13.2")
     session.install("-e", ".")
     os.makedirs("build", exist_ok=True)
     session.chdir("build")
@@ -312,7 +353,7 @@ def bundle_app(session: nox.Session) -> None:
         "./pyinstaller",
         "--onefile",
         "--collect-all",
-        "gcovr.writer",
+        "gcovr.formats",
         "-n",
         executable,
         *session.posargs,
@@ -321,7 +362,7 @@ def bundle_app(session: nox.Session) -> None:
     session.notify("check_bundled_app")
 
 
-@nox.session
+@nox.session(python=False)
 def check_bundled_app(session: nox.Session) -> None:
     """Run a smoke test with the bundled app, should not be used directly."""
     with session.chdir("build"):
@@ -343,7 +384,12 @@ def docker_container_os(session: nox.Session) -> str:
         return "ubuntu:18.04"
     elif session.env["CC"] in ["gcc-8", "gcc-9", "clang-10"]:
         return "ubuntu:20.04"
-    return "ubuntu:22.04"
+    elif session.env["CC"] in ["gcc-10", "gcc-11", "clang-13", "clang-14", "clang-15"]:
+        return "ubuntu:22.04"
+    elif session.env["CC"] in ["gcc-12", "gcc-13"]:
+        return "ubuntu:23.04"
+
+    raise RuntimeError(f"No container image defined for {session.env['CC']}")
 
 
 def docker_container_id(session: nox.Session, version: str) -> str:
@@ -354,22 +400,40 @@ def docker_container_id(session: nox.Session, version: str) -> str:
 @nox.session(python=False)
 def docker_build(session: nox.Session) -> None:
     """Build the docker container for the default GCC version."""
-    session_id = f"docker_build({GCC_VERSIONS[0]})"
+    session_id = f"docker_build({ALL_COMPILER_VERSIONS[0]})"
     session.log(f"Notify session {session_id}")
     session.notify(session_id)
 
 
 @nox.session(python=False, name="docker_build_compiler(all)")
 def docker_build_compiler_all(session: nox.Session) -> None:
+    """Build the docker containers vor all compiler versions."""
+    for version in ALL_COMPILER_VERSIONS:
+        session_id = f"docker_build_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_build_compiler(gcc)")
+def docker_build_compiler_gcc(session: nox.Session) -> None:
     """Build the docker containers vor all GCC versions."""
-    for version in GCC_VERSIONS:
+    for version in ALL_GCC_VERSIONS:
+        session_id = f"docker_build_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_build_compiler(clang)")
+def docker_build_compiler_clang(session: nox.Session) -> None:
+    """Build the docker containers vor all CLANG versions."""
+    for version in ALL_CLANG_VERSIONS:
         session_id = f"docker_build_compiler({version})"
         session.log(f"Notify session {session_id}")
         session.notify(session_id)
 
 
 @nox.session(python=False)
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def docker_build_compiler(session: nox.Session, version: str) -> None:
     """Build the docker container for a specific GCC version."""
     set_environment(session, version, False)
@@ -396,45 +460,63 @@ def docker_build_compiler(session: nox.Session, version: str) -> None:
 @nox.session(python=False)
 def docker_run(session: nox.Session) -> None:
     """Run the docker container for the default GCC version."""
-    session_id = f"docker_run_compiler({GCC_VERSIONS[0]})"
+    session_id = f"docker_run_compiler({ALL_COMPILER_VERSIONS[0]})"
     session.log(f"Notify session {session_id}")
     session.notify(session_id)
 
 
 @nox.session(python=False, name="docker_run_compiler(all)")
 def docker_run_compiler_all(session: nox.Session) -> None:
-    """Run the docker container for the all GCC versions."""
-    for version in GCC_VERSIONS:
+    """Run the docker container for the all compiler versions."""
+    for version in ALL_COMPILER_VERSIONS:
+        session_id = f"docker_run_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_run_compiler(gcc)")
+def docker_run_compiler_gcc(session: nox.Session) -> None:
+    """Run the docker containers vor all GCC versions."""
+    for version in ALL_GCC_VERSIONS:
+        session_id = f"docker_run_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_run_compiler(clang)")
+def docker_run_compiler_clang(session: nox.Session) -> None:
+    """Run the docker containers vor all CLANG versions."""
+    for version in ALL_CLANG_VERSIONS:
         session_id = f"docker_run_compiler({version})"
         session.log(f"Notify session {session_id}")
         session.notify(session_id)
 
 
 @nox.session(python=False)
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def docker_run_compiler(session: nox.Session, version: str) -> None:
     """Run the docker container for a specific GCC version."""
     set_environment(session, version, False)
 
-    def shell_join(args):
-        if sys.version_info >= (3, 8):
-            return shlex.join(args)
-        else:
-            # Code for join taken from Python 3.9
-            return " ".join(shlex.quote(arg) for arg in args)
-
     nox_options = session.posargs
+    if not session.interactive:
+        nox_options.insert(0, "--non-interactive")
     if session._runner.global_config.no_install:
         nox_options.insert(0, "--no-install")
     if session._runner.global_config.reuse_existing_virtualenvs:
         nox_options.insert(0, "--reuse-existing-virtualenvs")
+
     session.run(
         "docker",
         "run",
         "--rm",
-        "-t",
+        "-it" if session.interactive else "-t",
         "-e",
         "CC",
+        "-e",
+        "USE_COVERAGE",
+        "-e",
+        f"HOST_OS={platform.system()}",
         "-v",
         f"{os.getcwd()}:/gcovr",
         docker_container_id(session, version),
@@ -446,22 +528,40 @@ def docker_run_compiler(session: nox.Session, version: str) -> None:
 @nox.session(python=False)
 def docker(session: nox.Session) -> None:
     """Build and run the docker container for the default GCC version."""
-    session_id = f"docker_compiler({GCC_VERSIONS[0]})"
+    session_id = f"docker_compiler({ALL_COMPILER_VERSIONS[0]})"
     session.log(f"Notify session {session_id}")
     session.notify(session_id)
 
 
 @nox.session(python=False, name="docker_compiler(all)")
 def docker_compiler_all(session: nox.Session) -> None:
-    """Build and run the docker container for all GCC versions."""
-    for version in GCC_VERSIONS:
+    """Build and run the docker container for all compiler versions."""
+    for version in ALL_COMPILER_VERSIONS:
+        session_id = f"docker_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_compiler(gcc)")
+def docker_compiler_gcc(session: nox.Session) -> None:
+    """Build and run the docker containers vor all GCC versions."""
+    for version in ALL_GCC_VERSIONS:
+        session_id = f"docker_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_compiler(clang)")
+def docker_compiler_clang(session: nox.Session) -> None:
+    """Build and run the docker containers vor all CLANG versions."""
+    for version in ALL_CLANG_VERSIONS:
         session_id = f"docker_compiler({version})"
         session.log(f"Notify session {session_id}")
         session.notify(session_id)
 
 
 @nox.session(python=False)
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def docker_compiler(session: nox.Session, version: str) -> None:
     """Build and run the docker container for a specific GCC version."""
     session_id = "docker_build_compiler({})".format(version)
@@ -475,24 +575,65 @@ def docker_compiler(session: nox.Session, version: str) -> None:
 @nox.session(python=False)
 def docker_qa(session: nox.Session) -> None:
     """Run the session qa for the default GCC version."""
-    session_id = f"docker_qa_compiler({GCC_VERSIONS[0]})"
+    session_id = f"docker_qa_compiler({ALL_COMPILER_VERSIONS[0]})"
     session.log(f"Notify session {session_id}")
     session.notify(session_id)
 
 
 @nox.session(python=False, name="docker_qa_compiler(all)")
 def docker_qa_compiler_all(session: nox.Session) -> None:
+    """Run the session qa for all compiler versions."""
+    for version in ALL_COMPILER_VERSIONS:
+        session_id = f"docker_qa_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_qa_compiler(gcc)")
+def docker_qa_compiler_gcc(session: nox.Session) -> None:
     """Run the session qa for all GCC versions."""
-    for version in GCC_VERSIONS:
+    for version in ALL_GCC_VERSIONS:
+        session_id = f"docker_qa_compiler({version})"
+        session.log(f"Notify session {session_id}")
+        session.notify(session_id)
+
+
+@nox.session(python=False, name="docker_qa_compiler(clang)")
+def docker_qa_compiler_clang(session: nox.Session) -> None:
+    """Run the session qa for all CLANG versions."""
+    for version in ALL_CLANG_VERSIONS:
         session_id = f"docker_qa_compiler({version})"
         session.log(f"Notify session {session_id}")
         session.notify(session_id)
 
 
 @nox.session(python=False)
-@nox.parametrize("version", [nox.param(v, id=v) for v in GCC_VERSIONS])
+@nox.parametrize("version", [nox.param(v, id=v) for v in ALL_COMPILER_VERSIONS])
 def docker_qa_compiler(session: nox.Session, version: str) -> None:
     """Run the session qa for a specific GCC version."""
     session_id = f"docker_compiler({version})"
     session.log(f"Notify session {session_id}")
     session.notify(session_id, ["-s", "qa", "--", *session.posargs])
+
+
+@nox.session(python=False)
+def import_reference(session: nox.Session) -> None:
+    """Import reference data from ZIP generated in Github pipeline."""
+    if len(session.posargs) != 1:
+        session.error(
+            "Exact one ZIP file needed. Usage: nox -s import_reference -- file.zip"
+        )
+
+    def extract(fh_zip: zipfile.ZipFile):
+        for entry in fh_zip.filelist:
+            session.log(fh_zip.extract(entry, "gcovr/tests"))
+
+    with zipfile.ZipFile(session.posargs[0]) as fh_zip:
+        try:
+            zip_info_diff_zip = fh_zip.getinfo("diff.zip")
+            with fh_zip.open(zip_info_diff_zip) as fh_inner_zip:
+                seekable_buf = io.BytesIO(fh_inner_zip.read())
+                with zipfile.ZipFile(seekable_buf) as fh_diff_zip:
+                    extract(fh_diff_zip)
+        except KeyError:
+            extract(fh_zip)
